@@ -29,6 +29,7 @@ c      ****************
        subroutine sopen
        implicit none
 c              read control parameters
+#include  "ZepManager.h"
 #include  "ZsepManager.h"
 #include  "ZepDirec.h"
 #include  "ZepTrackp.h"
@@ -45,6 +46,7 @@ c
      *   cosmosparam
 
         call sparmr(SepicsFile)
+
 c
 c                   init epics
         if(cosmosparam .ne. ' ') then
@@ -52,6 +54,7 @@ c                   init epics
         endif
 
         call epiaev(EpicsFile, ConfigFile)
+
         if(Trace .and. Nevent .gt. 20) then
            call cerrorMsg('********* Warning **********',1)
            call cerrorMsg(
@@ -61,7 +64,17 @@ c                   init epics
      *     'If not, make "Trace f" in epicsfile', 1)
         endif
 c              incident energy sampling. initializaiton 
-        call ciniSPrim(PrimaryFile)
+        if( PrimaryFile(1:1) .eq. "+") then
+           call siniPrim(PrimaryFile)
+        else
+           Inpxyz = 0
+           Inpdir = 0
+           Inperg  = 0
+           Inptime = 0
+           call ciniSPrim(PrimaryFile)
+
+        endif
+
 c
 c            inquire form
         call epqfrm(formx)
@@ -70,6 +83,8 @@ c            inquire form
         elseif(formx .eq. 'pipe' ) then
            call epqpip(rpipi, rpipo)
         endif
+
+
 c        call epqcnf(orgw, abcw)
 
 c        call epqworld(nwld, wstruct) 
@@ -105,8 +120,152 @@ c         ---------------- Error -----------------
           write(msg,*) ' no control parameters'
           call cerrorMsg(msg, 0)
        end
+       subroutine  siniPrim(file)
+       implicit none
+#include  "ZepManager.h"
+       character(*) file
+       character*12  inpcond
+       integer klena,icon
+
+       open(Ioprim, file=file(1:klena(file) ))
+       read(Ioprim, '(a)')  inpcond
+       if( index(inpcond, "xyz") .gt. 0  ) then
+          Inpxyz = 1
+       else
+          Inpxyz = 0
+       endif
+       if( index(inpcond, "dir")  .gt. 0 ) then
+          Inpdir =  1
+       else
+          Inpdir = 0
+       endif
+       if( index(inpcond, "sub") .gt. 0 ) then
+          Inpsubcode = 1
+       else
+          Inpsubcode = 0
+       endif
+       if( index(inpcond, "KE") .gt.  0  .or.
+     *     index(inpcond, "ke") .gt.  0   ) then
+          Inperg = 1
+       elseif( index(inpcond, "TE") .gt. 0 .or.
+     *     index(inpcond, "te") .gt. 0  ) then
+          Inperg = 2
+       else
+          Inperg = 0
+       endif
+       if(index(inpcond, "time") .gt. 0 ) then
+          Inptime = 1
+       else
+          Inptime = 0
+       endif
+       call cskipComment(Ioprim, icon)
+       if(icon .ne. 0) then
+          call cerrorMsg("Primary file has no #----- line", 0)
+       endif
+       end
+       subroutine ssetip2( aTrack, icon)
+       implicit none
+#include  "ZepManager.h"
+#include  "ZepTrack.h"
+#include  "Zcode.h"
+       record /epTrack/ aTrack
+       integer icon, nf, fc
+       integer code, subcode, chg 
+       real*8 erg, xin, yin, zin, wx, wy, wz, u
+       character*80 buf
+       character*20 field(11)
+
+       buf = ' '
+       subcode = 0
+
+       read(Ioprim, '(a)', end= 100 )  buf
+
+       call kgetField(buf, field, 11, nf)
+
+       
+       fc = 1
+       read(field(fc), *) code
+       if(Inpsubcode .eq. 1) then
+          fc = fc + 1
+          read(field(fc), *)  subcode
+       endif
+       fc = fc + 1
+       read(field(fc), *)  chg
+       if(Inperg .gt. 0) then
+          fc = fc + 1 
+          read(field(fc), *)   erg
+       endif
+       if( Inpxyz .gt. 0 ) then
+          fc = fc + 1
+          read( field(fc), *)   xin
+          fc = fc + 1
+          read( field(fc), *)   yin
+          fc = fc + 1
+          read( field(fc), *)   zin
+       endif
+       if( Inpdir .gt. 0 ) then
+          fc = fc + 1
+          read( field(fc), *)   wx
+          fc = fc + 1
+          read( field(fc), *)   wy
+          fc = fc + 1
+          read( field(fc), *)   wz
+       endif
+       if( Inptime .gt. 0 ) then
+          fc = fc + 1
+          read( field(fc), *)   aTrack.t 
+          aTrack.t = aTrack.t*29.98d0    ! cm (t is nsec)
+       endif
+c         check number of field; some redandant field may exist at the last part, so
+c         nf many not be equal to fc. but must be nf >= fc
+       if(nf .lt.  fc) then
+          call cerrorMsg(
+     *      'No. of fields in primary data is too few', 1)
+          call cerrorMsg('data given is', 1)
+          call cerrorMsg(buf, 1)
+          stop 9999
+       endif
+
+       if(subcode .eq. 0 .and.  chg .eq. 0) then
+          call rndc(u)
+          if(code .eq. kkaon) then
+             if(u .lt. 0.5) then
+                subcode = k0l
+             else
+                subcode = k0s
+             endif
+          elseif(code .eq. knuc) then
+             if(u .lt. 0.5) then
+                subcode = antip
+             else
+                subcode = regptcl
+             endif
+          endif
+       endif
+
+       call cmkptc( code, subcode, chg, aTrack.p)
+       if(Inpxyz .eq. 1) then
+          aTrack.pos.x = xin
+          aTrack.pos.y = yin
+          aTrack.pos.z = zin
+       endif
+       if(Inpdir .eq. 1) then
+          aTrack.w.x = wx
+          aTrack.w.y = wy
+          aTrack.w.z = wz
+       endif
+       aTrack.p.fm.p(4)  =  erg
+       if(Inperg .eq. 1) then
+          aTrack.p.fm.p(4) =  aTrack.p.fm.p(4) + aTrack.p.mass
+       endif
+       icon = 0
+       return
+ 100   icon = 1
+       end
+
        subroutine  s1set(icon)
        implicit none
+#include  "ZepManager.h"
 #include  "ZsepManager.h"
 #include  "ZepPos.h"
 #include  "ZepDirec.h"
@@ -130,16 +289,23 @@ c             record Seed in err out
          endif
 c                 init 1 event. (clear arrays etc)
          call epi1ev(jcon)
+
+
          if(jcon .ne. 0) then
 c                    jcon=0-->user set the incident
 c                    set incident ptcl(s) by sepics
-            call ssetip
+
+            call ssetip(icon)
+         else
+            icon = 0       ! this is needed 2004. May 2
          endif
-         call stimec(icon)
-c                  check time using stack area data
          if(icon .eq. 0) then
-c                     time is available for the generation
-c                     generate 1 event
+c                  check time using stack area data
+            call stimec(icon)
+         endif
+         if(icon .eq. 0) then
+c              time and incident are available  for the generation
+c              generate 1 event
             call epgen
 c              end of 1 event
             call epe1ev
@@ -178,14 +344,15 @@ c      *****************
              icon=0
           endif
        end
-      subroutine ssetip
+      subroutine ssetip( icon )
       implicit none
+#include  "ZepManager.h"
 #include  "ZsepManager.h"
 #include  "ZepTrack.h"
 #include  "Zep3Vec.h"
 #include  "Zswk.h"
 #include  "Zsparm.h"
-
+      
       record /epTrack/ aTrack
       record /epTrack/InciTrack, bTrack
       integer ns, i, j, icon
@@ -193,13 +360,24 @@ c      *****************
 
 c             set incident ptcl
 c               fix energy (momentum yet)
-      call sfixe(aTrack)
+      if(Inperg .eq. 0) then
+         call sfixe(aTrack)
+         icon = 0
+      else
+         call ssetip2(aTrack, icon)
+         if(icon .ne. 0) return ! ************
+      endif
 
 c               fix position
-      call sfixp(aTrack)
+      if(Inpxyz .eq. 0) then
+         call sfixp(aTrack)
+      endif
 c               fix angle
-      call sfixa(aTrack)
+      if(Inpdir .eq. 0) then
+         call sfixa(aTrack)
+      endif
 c               fix others
+
       call sfixo(aTrack)
 c               set them
       call epputTrack(aTrack)
@@ -259,6 +437,7 @@ c             get  current random no.
        end
       subroutine sfixe(aTrack)
        implicit none
+#include  "ZepManager.h"
 #include  "Zglobalc.h"
 #include  "ZepTrack.h"
 #include  "ZepTrackp.h"
@@ -284,6 +463,7 @@ c             get  current random no.
 
 
 c           sample incident particle
+       
        call epsampPtcl(aTrack.p)
        return
 c      *******************
@@ -454,9 +634,11 @@ c              direct the angle around the normal vector
         endif
         return
 c        ***************************
-      entry sfixo(aTrack)
+        entry sfixo(aTrack)
 c              fix time of incidence
-        aTrack.t = 0.d0
+        if( Inptime .eq. 0) then
+           aTrack.t = 0.d0
+        endif
 c           fix momentum 
         call epe2p(aTrack)  ! direction & e--> px,py,pz
        end
